// mailutil.h
// Copyright 1997 Jarle Aase
// The basic code from this module is taken from
// mailto (Copyright (c) 1992 Bell Communications Research, Inc. (Bellcore))

class CMailUtil;

class CSMTPError
{
public:
	int error_code;

	void operator = (int code) {error_code = code; throw this;}
};

class CMailHeaderLine
{
public:
	~CMailHeaderLine();
	static CMailHeaderLine *AddHeader(const char *Tag, const char *Text, CMailHeaderLine *pFirst, int DoSeperate);
	static CMailHeaderLine *Find(const char *Tag, CMailHeaderLine *pFirst);
	void PrintHdr(FILE *OutFp);
	char *GetText() {return buffer;}

protected:
	CMailHeaderLine();
	void Add(const char *Text, int DoSeperate);
	
	const char *tag;
	char *buffer;
	CMailHeaderLine *next;
};

class CMailPart
{
public:
	CMailPart(CMailUtil *pFather);
	~CMailPart();
	int Create(const char *Text, int Type, int DoConvertCharset);
	void WriteContentTypeAndEncoding(FILE *fp);
	void WriteCtypeNicely(FILE *fp, char *ct);
	void fputsquoting(char *s, FILE *fp);
	void controlputc(char c);
	void TranslateInputToEncodedOutput(FILE *OutFile);
	void TranslateInputToEncodedOutput(FILE *InputFile, FILE *OutFile);
	void TranslateInputToEncodedOutput(const char *Text, FILE *OutFile);
	static void to64(FILE *infile, FILE *outfile);
	static void to64(const char *Text, FILE *outfile) ;
	static void output64chunk(int c1, int c2, int c3, int pads, FILE *outfile);
	void toqp(FILE *infile, FILE *outfile);
	void toqp(const char *Text, FILE *outfile) ;

	int istext;
	int isrich;
	char *content_type;
	int encoding_type_needed;
	char filename[L_tmpnam];
	char *CharacterSet;
	char *buffer;
	CMailPart *next, *prev, *FirstPart;
	char content_buffer[MAX_PATH +64];
	char content_disposition[MAX_PATH + 32];
	CMailUtil *pOrigin;

	enum // Types
	{
		ASCII_HEADER, // Header node
		ASCII_7BITUS, // Force 7 bit us
		ASCII_ISO8859, // Force ISO 8859
		ASCII_GENERIC, // Find out...
		ASCII_TEXTFILE_INCLUCE, // Textfile included
		BINARY, // Binary attachment, base 64
	};
};

class CMailUtil
{
public:
	CMailUtil();
	~CMailUtil();

	// High level calls
	void SetTo(const char *Name, const char *Email);
	void SetCc(const char * Name, const char *Email);
	void SetBcc(const char * Email);
	void SetPriority(int Pri);
	void SetOemToAnsi(int True) {convert_charset = True;}

	int AddHdrLine(const char *tag, const char *Text, int DoSeperate = 0);
	const char *HdrValue(const char *Tag, const char *Def = "(NULL)");
	int AddSegment(const char *Text, int Type);
	int WriteOutMessage(FILE *OutFile);
	void SetFrom(const char *From);
	void SetSubject(const char *Subject);
	void SetReturnPath(const char *ReturnPath);
	void AddDateHdr(const char *Text = NULL);
	const char *GetAdrList() { return AdrList; }
	CMailPart *GetFirstPart() {return FirstPart; }
	void SetServerMode(int Mode) {mode = Mode; }
	int GetServerMode() {return mode; }
	void SetMime(int False) {DoMime = False;}
	int UsingMime() {return DoMime;}

protected:
	void AddAdressee(const char *Tag, const char *Name, const char *Email);

public:
	static int DebugFlag;
	static int VerboseFlag;
	CSMTPError error_val;
	int mode; // SMPT, NNTP
	int DoMime;
	static char Organization[127];

	enum // Modes
	{
		MODE_SMTP,
		MODE_NNTP,
	};

public:
	// Direct access to SMTP server
	// Support for Winsock 1.1 and better
	

protected:
	int _WriteOutMessage(FILE *OutFile);

	int convert_charset;
	char boundary[120];
	CMailPart *FirstPart;
	CMailHeaderLine *FirstHeaderLine;
	char *AdrList; // Comma seperated list of RCPT recitoents
};

#if DO_SMTP

#define LOCAL_HOST_NAME_LEN 256

class CSMTPEmail : public CMailUtil
{
public:
	CSMTPEmail();
	~CSMTPEmail();
	void HandleWinsock(int Bool) {DoHandleWinsock = Bool;}
	int SendIt();
	unsigned long GetAddr (const char *szHost) ;
	void SetHost(const char *Name) {strcpy(Host, Name);}

	static int DoHandleWinsock; // Initialize and shutdown winsock
	static int NumObjects;
	int IsCGI;
	static int NetIsStarted;
	static char Host[127];
	static char NNTPHost[127];
	static char LocalHost[LOCAL_HOST_NAME_LEN];
	static char UserName[32]; // NNTP
	static char UserPwd[32]; // NNTP


protected:
	int CSMTPEmail::_SendIt();
	int Post(SOCKET sc, const char *msg);
	int Ack(SOCKET sc);
};


// Return values
enum
{
	SUCCESS,
	FAILED_TO_PARSE_ARGUMENTS,
	FAILED_TO_OPEN_MAILFILE,
	FAILED_TO_START_SOCKETS,
	FAILED_TO_RESOLVE_HOST,
	FAILED_TO_OBTAIN_SOCKET_HANDLE,
	FAILED_TO_CONNECT,
	FAILED_TO_SEND,
	FAILED_TO_RECEIVE,
	SERVER_ERROR,
	FAILED_TO_GET_HOSTNAME,
	OUT_OF_MEMORY,
	FAILED_TO_OPEN_TMP_FILE,
	UNKNOWN_ERROR
};


#endif // DO_SMTP


/* Codes for encoding_type_needed */
#define ENC_NONE 0
#define ENC_QP 1 /* quoted-printable */
#define ENC_B64 2 /* base64 */

#ifndef TZNAME_MAX
# define TZNAME_MAX	50	/* max size of timezone */
#endif

/* values for TZ_TYPE */
#define TZ_NONE		0	/* no character timezone support */
#define TZ_TM_NAME	1	/* use tm->tm_name */
#define TZ_TM_ZONE	2	/* use tm->tm_zone */
#define TZ_TZNAME	3	/* use tzname[] */
#define TZ_TIMEZONE	4	/* use timezone() */


char *arpadate(char *ud, char *b);
const char *mail_tmpnam(LPSTR Name);
const char *GetRegisteryString(HKEY MainKey, LPCSTR Path, char *buffer, int buflen);
char *GetContentType(LPCSTR FileName, char *buf);

