// wMailTo.cpp
//
// Simple command line mailer to demonstrate the
// MailUtil library

#define APPNAME "wMailTo"
#define VERSION "1.10a"
#define COPYRIGHT APPNAME" v."VERSION" Copyright 1997 by Jarle Aase\r\n\tGet a free copy from http://www.jgaa.com"

#define DO_SMTP 1 // Required

#include <stdio.h>
#include <sys/stat.h>
#include <windows.h>

#include "mailutil.h"


char *strip_fgets(const char *prompt1, const char *prompt2, char *buf, int len, FILE *fp);
int DoSetup();
int usage(LPCSTR Tag);
void SetGetVar(LPCSTR Tag, LPCSTR Text);
int Initialize();

// Error messages
LPSTR ErrorMessages[] =
{
	{"Success"},
	{"Bad arguments from form"},
	{"Unable to open temporary mail file for read"},
	{"FAILED_TO_START_SOCKETS"},
	{"FAILED_TO_RESOLVE_HOST"},
	{"FAILED_TO_OBTAIN_SOCKET_HANDLE"},
	{"FAILED_TO_CONNECT"},
	{"FAILED_TO_SEND"},
	{"FAILED_TO_RECEIVE"},
	{"SERVER_ERROR"},
	{"Failed to resolve the host IP name"},
	{"Out of memory"},
	{"Failed to open temporary file"},
	{"Unknown error"}
};


CSMTPEmail Email;
BOOL DotFlag = FALSE;
BOOL PauseFlag = TRUE;
LPCSTR myname;

struct LINES
{
	char *Text;
	struct LINES *Next;
} *FirstLine = NULL, *Attachments = NULL, *Textfiles = NULL;

void AddLine(LPCSTR Text, LINES **Origin = &FirstLine);
void AddLine(LPCSTR Text, LINES **Origin)
{
	struct LINES *pLine = *Origin;
	while(pLine && pLine->Next)
		pLine = pLine->Next;

	LINES *NewLine = new LINES;
	NewLine->Text = strdup(Text);
	NewLine->Next = NULL;

	if (pLine)
		pLine->Next = NewLine;
	else
		*Origin = NewLine;
}

main(int argc, LPSTR*argv)
{
	LPCSTR p;
	myname = *argv;
	FILE *InFp = NULL;

	if ((p = strrchr((LPCSTR)myname,'\\')) != NULL)
		myname = ++p;
	else
		myname = *argv;

	if (argc == 2 && !strcmp(argv[1], "-init"))
		return DoSetup();

	if (!Initialize())
		return -1;

	while(--argc)
	{
		p = *++argv;
		if (*p == '-')
		{
			switch(*++p)
			{
				case 'd': Email.DebugFlag = TRUE; break;
				case 'v': Email.VerboseFlag = TRUE; break;
				case 's': Email.SetSubject(p+1); break;
				case 'f': Email.SetFrom(p+1); break;
				case 'p': PauseFlag = TRUE; break;
				case 'C': Email.IsCGI = FALSE; break;
				case '.': DotFlag = TRUE; break;
				case 'h': return usage(NULL);
				case 'i': AddLine(p+1, &Attachments); break;
				case 't': AddLine(p+1, &Textfiles); break;
				case 'a': Email.SetOemToAnsi(1); break;
				case 'P': Email.SetPriority(atoi(p+1)); break;
				case '-': InFp = stdin; break;
				case 'm': Email.SetMime(0); break;
				case 'u': Email.SetServerMode(Email.MODE_NNTP); break;
				case 'o': strcpy(Email.Organization, p+1); break;
				case 'c':
					{
						char *pp = (char *)p +1;
						for(pp = strtok(pp, ","); pp; pp = strtok(NULL, ","))
						Email.SetCc(NULL, pp);
					}
					break;
				case 'b':
					{
						char *pp = (char *)p +1;
						for(pp = strtok(pp, ","); pp; pp = strtok(NULL, ","))
						Email.SetBcc(pp);
					}
					break;

				case 'x': Email.SetHost(p+1); break;
				default:
					usage(*argv);
			}
		}
		else
		{
			// Add adressee
			char *pp = (char *)p;
			for(pp = strtok(pp, ","); pp; pp = strtok(NULL, ","))
				Email.SetTo(NULL, pp);
		}
	}

	if (Attachments && !Email.UsingMime())
	{
		return usage("-m and -i can not be combined");
	}

	if (!Email.GetAdrList())
	{
		// Go interactive - no reciptients are specified
		Email.VerboseFlag = TRUE;
		if (!Textfiles)
			InFp = stdin;

		fprintf(stderr,"%s %s\n\nGoing interactive - no receivers are specified.\n",
			APPNAME, COPYRIGHT);

		char buf[127];
		int num;


		if (Email.mode == Email.MODE_NNTP)
		{
			for(num = 0;
				strip_fgets("Newsgroup", num ? "Add one more or leave blank" : NULL, buf, sizeof(buf), stdin);
				num++)
			{
				if (!*buf)
					break;

				Email.SetTo(NULL, buf);
			}
		}
		else
		{
			for(num = 0;
				strip_fgets("To", num ? "Add one more or leave blank" : NULL, buf, sizeof(buf), stdin);
				num++)
			{
				if (!*buf)
					break;

				Email.SetTo(NULL, buf);
			}

			for(num = 0;
				strip_fgets("Cc", num ? "Add one more or leave blank" : NULL, buf, sizeof(buf), stdin);
				num++)
			{
				if (!*buf)
					break;

				Email.SetCc(NULL, buf);
			}

			for(num = 0;
				strip_fgets("Bcc", num ? "Add one more or leave blank" : NULL, buf, sizeof(buf), stdin);
				num++)
			{
				if (!*buf)
					break;

				Email.SetBcc(buf);
			}
		}

		if (!*Email.HdrValue("Subject", ""))
		{
			strip_fgets("Subject", NULL, buf, sizeof(buf), stdin);
			if (*buf)
				Email.SetSubject(buf);
		}
	}

	if (!Email.GetAdrList())
	{
		if (Email.VerboseFlag)
			fprintf(stderr,"You must send the mail to someone...\n");

		return usage(NULL);
	}

	if (InFp)
	{
		if (DotFlag && Email.VerboseFlag)
			fprintf(stderr,"Enter message text. End with . (period) on a single line\n");
		else if (Email.VerboseFlag)
			fprintf(stderr,"Enter message text. End with ^Z\n");

		char buf[127];
		while(fgets(buf, sizeof(buf -1), InFp) != NULL)
		{
			p = buf;
			if (*p == '.')
			{
				if (DotFlag && ((p[1] == '\n') || (p[1] == '\r') || (!p[1])))
					break;

				memmove(buf +1, buf, strlen(buf) + 1);
				*buf = '.';
			}
			AddLine(buf);
		}
	}

	// Make buffer for bessage body
	int bodyLen = 3;
	struct LINES *pLine = FirstLine;

	while(pLine)
	{
		bodyLen += strlen(pLine->Text);
		pLine = pLine->Next;
	}

	// Add to buffer
	LPSTR body;
	LPSTR pp = body = (char *)malloc(bodyLen);
	for(pLine = FirstLine; pLine; pLine =  pLine->Next)
	{
		p = pLine->Text;
		while(*p)
			*pp++ = *p++;
	}

	*pp++ = '\r';
	*pp++ = '\n';
	*pp++ = 0;

	// Add text includes
	// We have to merge them into the primary body text, as
	// some mail readers (Eudora) don't handle multipart text
	// segments (dunno if that's according to the RFC...)
	for(pLine = Textfiles; pLine; pLine =  pLine->Next)
	{
		// This works fine, but mail-readers don't handle it :-(
		//Email.AddSegment(pLine->Text, CMailPart::ASCII_TEXTFILE_INCLUCE);
		struct _stat st;
		int buflen = 0;
		char *tmpbuf = NULL;
		int ch;
		unsigned char *p, *pp;

		if (_stat(pLine->Text, &st))
		{
			if (Email.VerboseFlag || Email.DebugFlag)
				fprintf(stderr,"%s: Text attachment file '%s' was not found. Aborting.\n",
					myname, pLine->Text);
			return -1;
		}

		buflen = st.st_size;

		FILE *fp = fopen(pLine->Text, "rb");
		if (!fp)
			return -1;

		tmpbuf = (char *)malloc(buflen + 1);

		p = (unsigned char *)tmpbuf;
		while((buflen-- > 0) && ((ch = getc(fp)) != EOF))
			*p++ = (unsigned char)ch;
		*p = 0;
		fclose(fp);

		// Calculate buffer size...
		buflen = 0;
		p = (unsigned char *)tmpbuf;
		while(*p)
		{
			if ((*p == '\n') && (((p > (unsigned char *)tmpbuf) && (p[-1] != '\r')) || (p[1] == '.')))
				buflen += 2;
			else
				buflen++;
			++p;
		}

		// Copy buffer
		p = (unsigned char *)tmpbuf;
		int oldlen = strlen(body);
		body = (char *)realloc(body, oldlen + buflen +1);
		pp = (unsigned char *)body + oldlen;

		while(*p)
		{
			if (*p == '\n' && ((p > (unsigned char *)tmpbuf) && (p[-1] != '\r')))
			{
				*pp++ = '\r';
				*pp++ = '\n';
				if (*++p == '.')
				{
					*pp++ = '.';
					*pp++ = *p++;
				}
			}
			else if ((p == (unsigned char *)tmpbuf) && (*p == '.'))
			{
				*pp++ = *p++;
				*pp++ = '.';
			}
			else if ((*p == '\n') && (p[1] == '.'))
			{
				*pp++ = *p++;
				*pp++ = *p++;
				*pp++ = '.';
			}
			else
				*pp++ = *p++;
		}
		*pp = 0;
		free(tmpbuf);
	}

	Email.AddSegment(body, CMailPart::ASCII_GENERIC);

	// Add attachments
	for(pLine = Attachments; pLine; pLine =  pLine->Next)
	{
		Email.AddSegment(pLine->Text, CMailPart::BINARY);
	}

	Email.AddHdrLine("X-Mailer", COPYRIGHT);

	// Send mail
	int Rval = Email.SendIt();
	if (Rval && Email.VerboseFlag)
		fprintf(stderr,"%s: Error: %s\n", myname, ErrorMessages[Rval]);
	else if (Email.VerboseFlag)
	{
		if (Email.mode == Email.MODE_NNTP)
			fprintf(stderr,"%s: Message was successfully posted.\n", myname);
		else
			fprintf(stderr,"%s: Email was successfully sent.\n", myname);
	}

	return Rval;
}


int usage(LPCSTR Tag)
{
	if (Tag && !Email.VerboseFlag)
		return -1;

	fprintf(stderr,"%s %s - donated into the Public Domain (almost) by Jarle Aase\n", APPNAME, VERSION);
	fprintf(stderr,"THIS PROGRAM IS NOT LISENCED TO GOVERNMENTAL OR MILITARY USE.\n");

	if (Tag)
	{
		fprintf(stderr,"%s: Unknown parameter\n", Tag);
	}
	else
	{
		fprintf(stderr,"Usage: %s [options] {to[,to] [to] | @addr_list_file}\n", myname);
		fprintf(stderr,"Usage: %s -- [options] {to[,to] [to] | @addr_list_file} < message\n", myname);
		fprintf(stderr,"Usage: %s -init  Install %s\n", myname, myname);
		fprintf(stderr,"Options:\n");
		fprintf(stderr,"  -.      will end a piped message when a line contains a single dot.\n");
		fprintf(stderr,"  --      read text from standard input (console or pipe)\n");
		fprintf(stderr,"  -d      turn on debug mode\n");
		fprintf(stderr,"  -sSubject  adds a subject to the message\n");
		fprintf(stderr,"  -v      turn on verbose (interactive) mode\n");
		fprintf(stderr,"  -p      pause after the message is sent\n");
		fprintf(stderr,"  -h      show this help\n");
		fprintf(stderr,"  -iFile  include a binary attachment\n");
		fprintf(stderr,"  -tFile  include a textfile in the message body\n");
		fprintf(stderr,"  -a      convert OEM (european DOS codepages) charset to ANSI\n");
		fprintf(stderr,"  -Pn     sets priority (where n is 1 = highest, 3 = normal, 5 = lowest)\n");
		fprintf(stderr,"  -u      post to usenet (news). Receiver is newsgroup.\n");
		fprintf(stderr,"  -cAddress Cc to Address.\n");
		fprintf(stderr,"  -c@FileName Cc to Addressees defined in file FileName\n");
		fprintf(stderr,"  -bAddress Bcc to Address.\n");
		fprintf(stderr,"  -b@FileName Bcc to Addressees defined in file FileName\n");
		fprintf(stderr,"  -oOrganization.\n");
		fprintf(stderr,"  -xSMTPServer Override SMTP server defined in ini file\n");
		fprintf(stderr,"\n\nYou can attach multiple files, and have multiple receivers\n");
		fprintf(stderr,"You can add receivers with their full name: \"Jarle Aase <jgaa@mail.jgaa.com>\"\n");
	}

	return -1;
}

int DoSetup()
{
	fprintf(stderr,"%s: Setup\r\n\r\n", myname);
	SetGetVar("HOST", "Your mail server");
	SetGetVar("Return Address", "Default return path (Your email address)");
	SetGetVar("Organization", "Organization (not required)");

	fprintf(stderr,"\nThe following applies only if you will use %s to\npost usenet messages.\n", myname);
	SetGetVar("NNTP HOST", "Your NNTP server");
	SetGetVar("NNTP User", "Login name on server");
	SetGetVar("NNTP Passws", "Login pawssword on server");

	return -1;
}

int Initialize()
{
	char buf[127];
	GetProfileString("Setup", "HOST", "", buf, sizeof(buf));
	if (!*buf)
		goto err;

	Email.SetHost(buf);

	GetProfileString("Setup", "Return Address", "", buf, sizeof(buf));
	if (!*buf)
		goto err;

	Email.SetReturnPath(buf);

	if (!*Email.UserName)
	{
		GetProfileString("Setup", "NNTP User", "", buf, sizeof(buf));
		strcpy(Email.UserName, buf);
	}

	if (!*Email.UserPwd)
	{
		GetProfileString("Setup", "NNTP Passws", "", buf, sizeof(buf));
		strcpy(Email.UserPwd, buf);
	}

	if (!*Email.NNTPHost)
	{
		GetProfileString("Setup", "NNTP HOST", "", buf, sizeof(buf));
		strcpy(Email.NNTPHost, buf);
	}

	if (!*Email.Organization)
	{
		GetProfileString("Setup", "Organization", "", buf, sizeof(buf));
		strcpy(Email.Organization, buf);
	}

	return TRUE;

err:
	fprintf(stderr,"%s: Not initialized.\r\nRun: %s -init\n\n", myname, myname);
	return FALSE;
}

void SetGetVar(LPCSTR Tag, LPCSTR Text)
{
	char buf[127];
	char defval[127];

	GetProfileString("Setup", Tag, "", defval, sizeof(buf));

	fprintf(stderr,"%s (%s): ", Text, defval);
	fgets(buf, sizeof(buf), stdin);
	if (!*buf || (*buf == '\n') || (*buf == '\r'))
		strcpy(buf, defval);

	WriteProfileString("Setup", Tag, buf);
}

char *strip_fgets(const char *prompt1, const char *prompt2, char *buf, int len, FILE *fp)
{
	if (prompt2)
		fprintf(stderr,"%s (%s): ", prompt1, prompt2);
	else if (prompt1)
		fprintf(stderr,"%s: ", prompt1);

	char *p = fgets(buf, len, fp);
	if (p)
	{
		while(*p)
		{
			if ((*p == '\r') || (*p == '\n'))
			{
				*p = 0;
				break;
			}
			++p;
		}
		return buf;
	}
	return NULL;
}
